package org.sonatype.plexus.jetty;

import java.io.File;
import java.io.IOException;

import org.codehaus.plexus.classworlds.ClassWorld;
import org.codehaus.plexus.interpolation.InterpolationException;
import org.tanukisoftware.wrapper.WrapperListener;
import org.tanukisoftware.wrapper.WrapperManager;

/**
 * Class extending Jetty7 and adding Java Service Wrapper support on it, making Jetty7 "jsw-ized". This class is used as
 * "main" launched class in Nexus bundle (JSW launches ClassWorlds, that launches this). Requirement for App is to have
 * path to jetty.xml passed as first argument.
 * 
 * @author cstamas
 */
public class Jetty7WrapperListener
    extends Jetty7
    implements WrapperListener
{
    protected static final Object waitObj = new Object();

    private Jetty7WrapperListener( final File jettyXml )
        throws InterpolationException, IOException
    {
        // nope, do not instantiate this directly, just from main()!
        super( jettyXml );
    }

    // WrapperListener
    public Integer start( final String[] args )
    {
        WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_INFO, "Starting Jetty..." );

        try
        {
            startJetty();

            return null;
        }
        catch ( Exception e )
        {
            WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_FATAL, "Unable to start Jetty: " + e.getMessage() );

            e.printStackTrace();

            return 1;
        }
    }

    // WrapperListener
    public int stop( final int exitCode )
    {
        WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_INFO, "Stopping Jetty..." );

        try
        {
            stopJetty();

            return exitCode;
        }
        catch ( Exception e )
        {
            WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_FATAL,
                "Unable to stop Jetty cleanly: " + e.getMessage() );

            e.printStackTrace();

            return 1;
        }
        finally
        {
            synchronized ( waitObj )
            {
                waitObj.notify();
            }
        }
    }

    // WrapperListener
    public void controlEvent( final int event )
    {
        if ( ( event == WrapperManager.WRAPPER_CTRL_LOGOFF_EVENT ) && WrapperManager.isLaunchedAsService() )
        {
            // Ignore this event, it's just user logged out and we are a service, so continue running
            if ( WrapperManager.isDebugEnabled() )
            {
                WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_DEBUG, "Jetty7WrapperListener: controlEvent("
                    + event + ") Ignored" );
            }
        }
        else
        {
            if ( WrapperManager.isDebugEnabled() )
            {
                WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_DEBUG, "Jetty7WrapperListener: controlEvent("
                    + event + ") Stopping" );
            }

            WrapperManager.stop( 0 );
            // Will not get here.
        }
    }

    // ==

    /**
     * "Standard" main method, starts embedded Jetty "by the book", and it returns.
     * 
     * @param args
     * @throws IOException
     * @throws InterpolationException
     */
    public static void main( final String[] args )
        throws InterpolationException, IOException
    {
        if ( args != null && args.length > 0 )
        {
            final File jettyXml = new File( args[0] );

            WrapperManager.start( new Jetty7WrapperListener( jettyXml ), args );
        }
        else
        {
            WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_FATAL,
                "First supplied app parameter should be path to existing jetty7 XML configuration file!" );

            WrapperManager.stop( 1 );
        }
    }

    /**
     * "Enhanced" main method, used by Plexus ClassWorlds, that starts Jetty but also blocks until it's stopped. It has
     * to block, since ClassWorlds Launcher class would issue System.exit() upon return from this method.
     * 
     * @param args
     * @param world
     * @throws IOException
     * @throws InterpolationException
     */
    public static void main( final String[] args, final ClassWorld world )
        throws InterpolationException, IOException
    {
        // Note: it seems the WrapperManager.log does not work until WrapperManager.start is invoked
        // Hence, this line below will NOT appear, but it is invoked, honest!
        WrapperManager.log( WrapperManager.WRAPPER_LOG_LEVEL_INFO,
            "Started using Classworld \"enhanced\" main method..." );

        main( args );

        // because of stupid ClassWorlds Launcher, we need NOT to exit
        // now wait forever.
        try
        {
            synchronized ( waitObj )
            {
                waitObj.wait();
            }
        }
        catch ( InterruptedException e )
        {
        }
    }
}
